@@ -33,10 +33,10 @@ class Measurement(NamedTuple):
 
     def with_maximum(self, width: int) -> "Measurement":
         """Get a RenderableWith where the widths are <= width.
-        
+
         Args:
             width (int): Maximum desired width.
-        
+
         Returns:
             RenderableWidth: new RenderableWidth object.
         """
@@ -52,7 +52,7 @@ class Measurement(NamedTuple):
         Args:
             console (~rich.console.Console): Console instance.
             renderable (RenderableType): An object that may be rendered with Rich.
-            max_width (int, optional): The maximum width available, or None to use console.width. 
+            max_width (int, optional): The maximum width available, or None to use console.width.
                 Defaults to None.
 
         Raises:
@@ -64,6 +64,8 @@ class Measurement(NamedTuple):
         from rich.console import RichCast
 
         _max_width = console.width if max_width is None else max_width
+        if _max_width < 1:
+            return Measurement(0, 0)
         if isinstance(renderable, str):
             renderable = console.render_str(renderable)
 
@@ -78,9 +80,11 @@ class Measurement(NamedTuple):
                     .normalize()
                     .with_maximum(_max_width)
                 )
+                if render_width.maximum < 1:
+                    return Measurement(0, 0)
                 return render_width.normalize()
             else:
-                return Measurement(1, _max_width)
+                return Measurement(0, _max_width)
         else:
             raise errors.NotRenderableError(
                 f"Unable to get render width for {renderable!r}; "
@@ -102,7 +106,8 @@ def measure_renderables(
         Measurement: Measurement object containing range of character widths required to
         contain all given renderables.
     """
-
+    if not renderables:
+        return Measurement(0, 0)
     get_measurement = Measurement.get
     measurements = [
         get_measurement(console, renderable, max_width) for renderable in renderables
